{% extends 'layout.html' %}
{% block content %}
<div class="container">
  <div class="row">
    <div class="col-md-12">
      <div style="padding:100px 15px; text-align:center;">
        <div class="error-details">
          <h3>Password</h3>
        </div>

        <form class='mt-4' method="post">

          <input class='hvr-grow' size="40" style="border-top:0; border-left:0;border-right:0;"
            placeholder='Enter Password' id='mypass' type='password' name="text">
          <br>
          <input class='mt-4 btn-lg btn-info' type="submit">
        </form>

      </div>
    </div>
  </div>
</div>
{% endblock %}